@@ -13,13 +13,13 @@
 #include "entrylist.h"
 #include "kanjibrowser.h"
 #include "kanjibrowserconfig.h"
+#include "searchdialog.h"
 
 #include <KActionCollection>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <QAction>
 #include <QClipboard>
-#include <QDebug>
 
 KanjiBrowserView::KanjiBrowserView(QWidget *parent)
     : QWidget(parent)
@@ -109,13 +109,20 @@ void KanjiBrowserView::changeToListPage()
 QString KanjiBrowserView::convertToCSS(const QFont &font)
 {
     QString weight;
+    // TODO: these have numeric values in Qt we could plug into the CSS here
     switch (font.weight()) {
+    case QFont::Thin:
+    case QFont::ExtraLight:
     case QFont::Light:
         weight = QStringLiteral("lighter");
         break;
+    case QFont::Medium:
     case QFont::Normal:
         weight = QStringLiteral("normal");
         break;
+    case QFont::DemiBold:
+    case QFont::ExtraBold:
+    case QFont::Black:
     case QFont::Bold:
         weight = QStringLiteral("bold");
         break;
@@ -134,7 +141,7 @@ QString KanjiBrowserView::convertToCSS(const QFont &font)
         break;
     }
 
-    return QString(
+    return QStringLiteral(
                "font-family:\"%1\";"
                "font-size:%2px;"
                "font-weight:%3;"
@@ -177,21 +184,23 @@ void KanjiBrowserView::reloadKanjiList()
     statusBarChanged(i18np("%1 kanji found", "%1 kanji found", _kanjiList->count()));
 }
 
-void KanjiBrowserView::searchKanji(QListWidgetItem *item)
+void KanjiBrowserView::searchKanji(const QString &term)
 {
-    if (_currentKanji != nullptr && item->text() == _currentKanji->getWord()) {
+    if (_currentKanji != nullptr && term == _currentKanji->getWord()) {
         return;
     }
 
-    _goToKanjiInfo->setText(i18n("About %1", item->text()));
-    _copyToClipboard->setText(i18n("Copy %1 to clipboard", item->text()));
-    _copyToClipboard->setVisible(true);
+    if (auto result = _parent->_dictFileKanjidic->doSearch(DictQuery(term)); result != nullptr && !result->isEmpty()) {
+        auto kanji = dynamic_cast<EntryKanjidic *>(result->first());
+        _currentKanji = kanji;
 
-    Entry *result = _parent->_dictFileKanjidic->doSearch(DictQuery(item->text()))->first();
-    auto kanji = static_cast<EntryKanjidic *>(result);
-    _currentKanji = kanji;
+        _goToKanjiInfo->setText(i18n("About %1", _currentKanji->getWord()));
+        _copyToClipboard->setText(i18n("Copy %1 to clipboard", _currentKanji->getWord()));
+        _copyToClipboard->setVisible(true);
 
-    showKanjiInformation(kanji);
+        showKanjiInformation(kanji);
+        _goToKanjiInfo->triggered();
+    }
 }
 
 void KanjiBrowserView::setupView(KanjiBrowser *parent, const QHash<QString, QPair<int, int>> &kanji, QList<int> &kanjiGrades, QList<int> &strokeCount)
@@ -214,6 +223,10 @@ void KanjiBrowserView::setupView(KanjiBrowser *parent, const QHash<QString, QPai
     _goToKanjiInfo = _parent->actionCollection()->addAction(QStringLiteral("kanji_info"));
     _goToKanjiInfo->setText(i18n("Kanji &Information"));
 
+    _searchKanjiAction = _parent->actionCollection()->addAction(QStringLiteral("search"));
+    _searchKanjiAction->setIcon(QIcon::fromTheme(QStringLiteral("search")));
+    _searchKanjiAction->setText(i18n("&Search…"));
+
     _copyToClipboard = _parent->actionCollection()->addAction(QStringLiteral("copy_kanji_to_clipboard"));
     _copyToClipboard->setVisible(false);
 
@@ -235,11 +248,16 @@ void KanjiBrowserView::setupView(KanjiBrowser *parent, const QHash<QString, QPai
 
     connect(_grades, static_cast<void (KComboBox::*)(int)>(&KComboBox::currentIndexChanged), this, &KanjiBrowserView::changeGrade);
     connect(_strokes, static_cast<void (KComboBox::*)(int)>(&KComboBox::currentIndexChanged), this, &KanjiBrowserView::changeStrokeCount);
-    connect(_kanjiList, &QListWidget::itemClicked, this, &KanjiBrowserView::searchKanji);
-    connect(_kanjiList, &QListWidget::itemClicked, _goToKanjiInfo, &QAction::triggered);
+    connect(_kanjiList, &QListWidget::itemClicked, this, [this](QListWidgetItem *item) {
+        searchKanji(item->text());
+    });
+    connect(_kanjiList, &QListWidget::itemClicked, _goToKanjiInfo, [this] {
+        _goToKanjiInfo->triggered();
+    });
     connect(goToKanjiList, &QAction::triggered, this, &KanjiBrowserView::changeToListPage);
     connect(_goToKanjiInfo, &QAction::triggered, this, &KanjiBrowserView::changeToInfoPage);
     connect(_copyToClipboard, &QAction::triggered, this, &KanjiBrowserView::toClipboard);
+    connect(_searchKanjiAction, &QAction::triggered, this, &KanjiBrowserView::openSearchDialog);
 
     // Set the current grade (Grade 1).
     _grades->setCurrentIndex(1);
@@ -260,17 +278,17 @@ void KanjiBrowserView::showKanjiInformation(const EntryKanjidic *kanji)
     kanjiFont.setPointSizeF(_kanjiSize.toReal());
 
     QString text;
-    text.append("<html><body><style>");
+    text.append(QStringLiteral("<html><body><style>"));
     text.append(QStringLiteral(".kanji { %1 }").arg(convertToCSS(kanjiFont)));
     text.append(QStringLiteral(".label { %1 }").arg(convertToCSS(_labelFont)));
     text.append(QStringLiteral(".kana  { %1 }").arg(convertToCSS(_kanaFont)));
-    text.append("</style>");
+    text.append(QStringLiteral("</style>"));
 
     // Put the kanji.
     text.append(QStringLiteral("<table><tr><td><p class=\"kanji\">%1</p></td>").arg(kanji->getWord()));
 
     // Now the kanji grades and number of strokes.
-    text.append("<td>");
+    text.append(QStringLiteral("<td>"));
     if (!kanji->getKanjiGrade().isEmpty()) {
         text.append(QStringLiteral("<p class=\"label\">%1 %2</p></br>").arg(i18n("Grade:")).arg(kanji->getKanjiGrade()));
     }
@@ -278,38 +296,38 @@ void KanjiBrowserView::showKanjiInformation(const EntryKanjidic *kanji)
 
     // Onyomi readings.
     if (!kanji->getOnyomiReadingsList().isEmpty()) {
-        text.append(QString("<p class=\"label\">%1"
-                            "<span class=\"kana\">%2</span></p></br>")
+        text.append(QStringLiteral("<p class=\"label\">%1"
+                                   "<span class=\"kana\">%2</span></p></br>")
                         .arg(i18n("Onyomi: "))
                         .arg(kanji->getOnyomiReadings()));
     }
 
     // Kunyomi readings.
     if (!kanji->getKunyomiReadingsList().isEmpty()) {
-        text.append(QString("<p class=\"label\">%1"
-                            "<span class=\"kana\">%2</span></p></br>")
+        text.append(QStringLiteral("<p class=\"label\">%1"
+                                   "<span class=\"kana\">%2</span></p></br>")
                         .arg(i18n("Kunyomi: "))
                         .arg(kanji->getKunyomiReadings()));
     }
 
     // Special readings used in names.
     if (!kanji->getInNamesReadingsList().isEmpty()) {
-        text.append(QString("<p class=\"label\">%1"
-                            "<span class=\"kana\">%2</span></p></br>")
+        text.append(QStringLiteral("<p class=\"label\">%1"
+                                   "<span class=\"kana\">%2</span></p></br>")
                         .arg(i18n("In names: "))
                         .arg(kanji->getInNamesReadings()));
     }
 
     // Reading used as radical.
     if (!kanji->getAsRadicalReadingsList().isEmpty()) {
-        text.append(QString("<p class=\"label\">%1"
-                            "<span class=\"kana\">%2</span></p></br>")
+        text.append(QStringLiteral("<p class=\"label\">%1"
+                                   "<span class=\"kana\">%2</span></p></br>")
                         .arg(i18n("As radical: "))
                         .arg(kanji->getAsRadicalReadings()));
     }
 
     // Meanings
-    text.append("<p class=\"label\">");
+    text.append(QStringLiteral("<p class=\"label\">"));
     if (kanji->getMeaningsList().count() == 1) {
         text.append(i18n("Meaning: "));
     } else {
@@ -318,7 +336,7 @@ void KanjiBrowserView::showKanjiInformation(const EntryKanjidic *kanji)
     text.append(QStringLiteral("<span class=\"kana\">%1</span></p>").arg(kanji->getMeanings()));
 
     // Close remaining tags and set the HTML text.
-    text.append("</body></html>");
+    text.append(QStringLiteral("</body></html>"));
     _kanjiInformation->setHtml(text);
 }
 
@@ -329,4 +347,14 @@ void KanjiBrowserView::toClipboard()
     cb->setText(_currentKanji->getWord(), QClipboard::Selection);
 }
 
+void KanjiBrowserView::openSearchDialog()
+{
+    auto dialog = new SearchDialog(this);
+    connect(dialog, &SearchDialog::search, this, [this, dialog](const QString &query) {
+        searchKanji(query);
+        dialog->close();
+    });
+    dialog->show();
+}
+
 #include "moc_kanjibrowserview.cpp"
